package com.diagbot.aggregate;

import com.diagbot.facade.TestFacade;
import com.diagbot.vo.TestLineVO;
import io.github.lvyahui8.spring.annotation.DataConsumer;
import io.github.lvyahui8.spring.annotation.DataProvider;
import io.github.lvyahui8.spring.annotation.InvokeParameter;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.web.multipart.MultipartFile;

import java.util.LinkedHashMap;
import java.util.Map;

/**
 * @Description: 提醒类聚合
 * @author: zhoutg
 * @time: 2019/10/16 13:37
 */
@Component
@Slf4j
public class IndicationTestAggregate {

    @Autowired
    TestFacade testFacade;

    @DataProvider("testAll")
    public Map<String, Map<String, Object>> testAll(
            @InvokeParameter("mapPath") Map<String, String> mapPath,
            @DataConsumer("t1") Map<String, Map<String, Object>> t1Map,
            @DataConsumer("t2") Map<String, Map<String, Object>> t2Map,
            @DataConsumer("t3") Map<String, Map<String, Object>> t3Map,
            @DataConsumer("t4") Map<String, Map<String, Object>> t4Map) {
        Map<String, Map<String, Object>> map = new LinkedHashMap<>();
        if (t1Map != null) {
            map.putAll(t1Map);
        }
        if (t2Map != null) {
            map.putAll(t2Map);
        }
        if (t3Map != null) {
            map.putAll(t3Map);
        }
        if (t4Map != null) {
            map.putAll(t4Map);
        }
        return map;
    }

    @DataProvider("t1")
    public Map<String, Object> t1(@InvokeParameter("mapPath") Map<String, String> mapPath) {
        Map<String, Object> map = new LinkedHashMap<>();
        MultipartFile file = null;

        file = testFacade.getMulFileByPath(mapPath.get("【开单合理性_输血】"));
        Map<String, Object> billTransfusionMap = testFacade.testIndicationTransfusion(file, new TestLineVO());
        map.put("【开单合理性_输血】",billTransfusionMap);

        file = testFacade.getMulFileByPath(mapPath.get("【危急值_化验】"));
        Map<String, Object> criticalMapLis = testFacade.importCriticalLisExcel(file, new TestLineVO());
        map.put("【危急值_化验】",criticalMapLis);

        file = testFacade.getMulFileByPath(mapPath.get("【危急值_辅检】"));
        Map<String, Object> criticalMapPacs = testFacade.importCriticalPacsExcel(file, new TestLineVO());
        map.put("【危急值_辅检】",criticalMapPacs);

        file = testFacade.getMulFileByPath(mapPath.get("【其他值提醒_化验】"));
        Map<String, Object> otherLisMap = testFacade.testOtherTipLis(file, new TestLineVO());
        map.put("【其他值提醒_化验】",otherLisMap);

        file = testFacade.getMulFileByPath(mapPath.get("【其他值提醒_辅检】"));
        Map<String, Object> otherPacsMap = testFacade.testOtherTipPacs(file, new TestLineVO());
        map.put("【其他值提醒_辅检】",otherPacsMap);

        file = testFacade.getMulFileByPath(mapPath.get("【其他值提醒_输血】"));
        Map<String, Object> otherTransfusionMap = testFacade.testOtherTipTransfusion(file, new TestLineVO());
        map.put("【其他值提醒_输血】",otherTransfusionMap);

        return map;
    }

    @DataProvider("t2")
    public Map<String, Object> t2(@InvokeParameter("mapPath") Map<String, String> mapPath) {
        Map<String, Object> map = new LinkedHashMap<>();
        MultipartFile file = testFacade.getMulFileByPath(mapPath.get("【高危药品】"));
        Map<String, Object> highDrugMap = testFacade.testHighRiskDrugExcel(file, new TestLineVO());
        map.put("【高危药品】",highDrugMap);

        file = testFacade.getMulFileByPath(mapPath.get("【高危手术整合】"));
        Map<String, Object> highOperationMap = testFacade.testHighRiskOperationExcel(file, new TestLineVO());
        map.put("【高危手术】",highOperationMap);

        return map;
    }

    @DataProvider("t3")
    public Map<String, Object> t3(@InvokeParameter("mapPath") Map<String, String> mapPath) {
        Map<String, Object> map = new LinkedHashMap<>();
        MultipartFile file = testFacade.getMulFileByPath(mapPath.get("【高危手术整合】"));

        TestLineVO testLineVO = new TestLineVO();
        testLineVO.setGetOne(true);
        Map<String, Object> highOperationComplexMap = testFacade.testHighRiskProcess(file, testLineVO);
        map.put("【高危手术复杂情况】",highOperationComplexMap);
        return map;
    }

    @DataProvider("t4")
    public Map<String, Object> t4(@InvokeParameter("mapPath") Map<String, String> mapPath) {
        Map<String, Object> map = new LinkedHashMap<>();
        MultipartFile file = null;
        file = testFacade.getMulFileByPath(mapPath.get("【开单合理性】"));
        TestLineVO testLineVO = new TestLineVO();
        testLineVO.setGetOne(true);
        Map<String, Object> billMap = testFacade.importExcel(file, testLineVO);
        map.put("【开单合理性】",billMap);

        return map;
    }
}
