package com.diagbot.process;

import com.diagbot.biz.push.entity.Item;
import com.diagbot.dto.BillMsg;
import com.diagbot.dto.BillNeoDTO;
import com.diagbot.dto.BillNeoMaxDTO;
import com.diagbot.dto.IndicationDTO;
import com.diagbot.dto.WordCrfDTO;
import com.diagbot.enums.NeoEnum;
import com.diagbot.enums.TypeEnum;
import com.diagbot.facade.NeoFacade;
import com.diagbot.model.entity.Clinical;
import com.diagbot.model.entity.Operation;
import com.diagbot.model.label.ChiefLabel;
import com.diagbot.model.label.PacsLabel;
import com.diagbot.model.label.PastLabel;
import com.diagbot.model.label.PresentLabel;
import com.diagbot.rule.AgeRule;
import com.diagbot.rule.CommonRule;
import com.diagbot.rule.DrugRule;
import com.diagbot.rule.GroupRule;
import com.diagbot.rule.LisRule;
import com.diagbot.rule.MedEquRule;
import com.diagbot.rule.PacsRule;
import com.diagbot.rule.SexRule;
import com.diagbot.rule.VitalRule;
import com.diagbot.util.BeanUtil;
import com.diagbot.util.CoreUtil;
import com.diagbot.util.ListUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.List;

/**
 * @Description: 开单总入口
 * @author: zhoutg
 * @time: 2018/8/6 9:11
 */
@Component
public class BillProcess {

    @Autowired
    NeoFacade neoFacade;
    @Autowired
    DrugRule drugRule;
    @Autowired
    SexRule sexRule;
    @Autowired
    AgeRule ageRule;
    @Autowired
    CommonRule commonRule;
    @Autowired
    LisRule lisRule;
    @Autowired
    PacsRule pacsRule;
    @Autowired
    VitalRule vitalRule;
    @Autowired
    GroupRule groupRule;
    @Autowired
    MedEquRule medEquRule;

    public void process(List<BillNeoDTO> billNeoDTOList, WordCrfDTO wordCrfDTO, IndicationDTO res) {
        // 合并图谱数据到同一个对象中
        List<BillNeoMaxDTO> billNeoMaxDTOList = new ArrayList<>();
        for (BillNeoDTO billNeoDTO : billNeoDTOList) {
            BillNeoMaxDTO billNeoMaxDTO = new BillNeoMaxDTO();
            if (billNeoDTO.getDrugBillNeoDTO() != null) {
                BeanUtil.copyProperties(billNeoDTO.getDrugBillNeoDTO(), billNeoMaxDTO);
                billNeoMaxDTO.setType(TypeEnum.drug.getName());
            } else if (billNeoDTO.getLisBillNeoDTO() != null) {
                BeanUtil.copyProperties(billNeoDTO.getLisBillNeoDTO(), billNeoMaxDTO);
                billNeoMaxDTO.setType(TypeEnum.lis.getName());
            } else if (billNeoDTO.getPacsBillNeoDTO() != null) {
                BeanUtil.copyProperties(billNeoDTO.getPacsBillNeoDTO(), billNeoMaxDTO);
                billNeoMaxDTO.setType(TypeEnum.pacs.getName());
            } else if (billNeoDTO.getOperationBillNeoDTO() != null) {
                BeanUtil.copyProperties(billNeoDTO.getOperationBillNeoDTO(), billNeoMaxDTO);
                if (ListUtil.isNotEmpty(billNeoDTO.getOperationBillNeoDTO().getMedallegen())) {
                    billNeoMaxDTO.getAllergicmeds().addAll(billNeoDTO.getOperationBillNeoDTO().getMedallegen());
                }
                billNeoMaxDTO.setType(TypeEnum.operation.getName());
            } else if (billNeoDTO.getTransfusionBillNeoDTO() != null) {
                BeanUtil.copyProperties(billNeoDTO.getTransfusionBillNeoDTO(), billNeoMaxDTO);
                billNeoMaxDTO.setType(TypeEnum.transfusion.getName());
            }
            billNeoMaxDTO.setOrderName(billNeoDTO.getName()); // 开单名称
            billNeoMaxDTO.setOrderStandName(billNeoDTO.getStandname()); // 开单标准名称
            // // TODO 测试数据开始
            // if (billNeoDTO.getStandname().equals("胸部CT")) {
            //     NodeNeoDTO nodeNeoDTO = new NodeNeoDTO();
            //     nodeNeoDTO.setName("幼儿");
            //     billNeoMaxDTO.getGroup().add(nodeNeoDTO);
            // }
            // if (billNeoDTO.getStandname().equals("胸部CT")) {
            //     NodeNeoDTO nodeNeoDTO = new NodeNeoDTO();
            //     nodeNeoDTO.setName("心电图");
            //     billNeoMaxDTO.getPacsOrder().add(nodeNeoDTO);
            // }
            // if (billNeoDTO.getStandname().equals("心电图")) {
            //     NodeNeoDTO nodeNeoDTO = new NodeNeoDTO();
            //     nodeNeoDTO.setName("胸部CT");
            //     billNeoMaxDTO.getPacsOrder().add(nodeNeoDTO);
            // }
            // if (billNeoDTO.getName().equals("普通胃镜检查")) {
            //     NodeNeoDTO sexNeo = new NodeNeoDTO();
            //     sexNeo.setName("男");
            //     billNeoMaxDTO.setGender(sexNeo);
            //
            //     NodeNeoDTO nodeNeoDTO = new NodeNeoDTO();
            //     nodeNeoDTO.setName("红细胞压积");
            //     nodeNeoDTO.setVal("阳性");
            //     billNeoDTO.getPacsBillNeoDTO().getLis().add(nodeNeoDTO);
            //
            //     NodeNeoDTO nodepacsDTO = new NodeNeoDTO();
            //     nodepacsDTO.setName("上腹部平扫");
            //     billNeoDTO.getPacsBillNeoDTO().getPacs().add(nodepacsDTO);
            //
            //     NodeNeoDTO opereat = new NodeNeoDTO();
            //     opereat.setName("胸腔镜下左下肺叶切除术");
            //     billNeoDTO.getPacsBillNeoDTO().getOperations().add(opereat);
            //     NodeNeoDTO opereat2 = new NodeNeoDTO();
            //     opereat2.setName("区域淋巴结清扫术");
            //     billNeoDTO.getPacsBillNeoDTO().getOperations().add(opereat2);
            //
            //     NodeNeoDTO drug = new NodeNeoDTO();
            //     drug.setName("青霉素类");
            //     billNeoDTO.getPacsBillNeoDTO().getAllergicmeds().add(drug);
            //
            //     NodeNeoDTO drug3 = new NodeNeoDTO();
            //     drug3.setName("泰舒达类");
            //     billNeoDTO.getPacsBillNeoDTO().getOralmeds().add(drug3);
            //
            //     // NodeNeoDTO vital = new NodeNeoDTO();
            //     // vital.setName("体温");
            //     // vital.setMax(new BigDecimal("39.1"));
            //     // vital.setMin(new BigDecimal("35.1"));
            //     // vital.setValType(1);
            //     // billNeoDTO.getPacsBillNeoDTO().getVitals().add(vital);
            //
            //     NodeNeoDTO vitalBp = new NodeNeoDTO();
            //     vitalBp.setName("舒张压");
            //     vitalBp.setMax(new BigDecimal("120"));
            //     vitalBp.setMin(new BigDecimal("100"));
            //     vitalBp.setValType(1);
            //     billNeoDTO.getPacsBillNeoDTO().getVitals().add(vitalBp);
            //
            //     NodeNeoDTO vitalBp2 = new NodeNeoDTO();
            //     vitalBp2.setName("收缩压");
            //     vitalBp2.setMax(new BigDecimal("120"));
            //     vitalBp2.setMin(new BigDecimal("100"));
            //     vitalBp2.setValType(1);
            //     billNeoDTO.getPacsBillNeoDTO().getVitals().add(vitalBp2);
            // }
            // 测试数据结束

            System.out.println(billNeoMaxDTO);
            billNeoMaxDTOList.add(billNeoMaxDTO);
        }
        processRule(billNeoMaxDTOList, wordCrfDTO, res);
    }

    // 规则处理
    public void processRule(List<BillNeoMaxDTO> billNeoMaxDTOList, WordCrfDTO wordCrfDTO, IndicationDTO indicationDTO) {
        ChiefLabel chiefLabel = wordCrfDTO.getChiefLabel();
        PresentLabel presentLabel = wordCrfDTO.getPresentLabel();
        PastLabel pastLabel = wordCrfDTO.getPastLabel();

        // 【过敏药品数据来源】
        List<Item> allergyMedicines = commonRule.getDrugAllergySource(wordCrfDTO);

        // 【药品数据来源】(主诉、现病史、既往史、结构化药品)
        List<Item> medicineAll = wordCrfDTO.getDrugSource();

        // 【诊断数据来源】
        List<Item> diags = wordCrfDTO.getDiagSource();

        // 【辅检结果数据来源】
        PacsLabel pacsLabel = wordCrfDTO.getPacsLabel();
        List<Item> pacsDescList = pacsLabel.getRes();

        // 【手术数据来源】
        List<Operation> operations_all = new ArrayList<>();
        List<Item> operation = wordCrfDTO.getOperation();
        List<Operation> operationsItem = new ArrayList<>();
        for (Item item : operation) {
            Operation operaData = new Operation();
            operaData.setName(item.getName());
            operaData.setStandName(item.getUniqueName());
            operationsItem.add(operaData);
        }
        CoreUtil.addList(operations_all, chiefLabel.getOperations()); // 主诉手术
        CoreUtil.addList(operations_all, presentLabel.getOperations()); // 现病史手术
        CoreUtil.addList(operations_all, pastLabel.getOperations()); // 既往史手术
        CoreUtil.addList(operations_all, operationsItem); // 结构化手术

        // 【临床表现数据来源】（主诉、现病史）
        List<Clinical> clinicals = new ArrayList<>();
        CoreUtil.addList(clinicals, chiefLabel.getClinicals()); // 主诉临床表现
        CoreUtil.addList(clinicals, presentLabel.getClinicals()); // 现病史临床表现

        List<BillMsg> billMsgList = new ArrayList<>();
        for (BillNeoMaxDTO bill : billNeoMaxDTOList) {
            // 性别
            sexRule.bill(wordCrfDTO, bill, billMsgList, NeoEnum.gender.getName());

            // 年龄
            ageRule.bill(wordCrfDTO, bill, billMsgList, NeoEnum.ageNeoDTO.getName());

            // 诊断
            commonRule.compareItemWithBill(bill.getDisease(), diags, bill, billMsgList, NeoEnum.disease.getName());

            // 化验
            lisRule.bill(wordCrfDTO.getLis(), bill, billMsgList, NeoEnum.lis.getName());

            // 体征
            vitalRule.bill(wordCrfDTO.getVitalLabel(), bill, billMsgList, NeoEnum.vitals.getName());

            // 辅检
            pacsRule.bill(bill.getPacs(), wordCrfDTO.getPacs(), bill, billMsgList, NeoEnum.pacs.getName());

            // 临床表现
            commonRule.compareNameWithBill(bill.getClinicfindings(), clinicals, bill, billMsgList, NeoEnum.clinicfindings.getName());

            // 手术(既往史、现病史、医嘱手术)
            commonRule.compareNameWithBill(bill.getOperations(), operations_all, bill, billMsgList, NeoEnum.operations.getName());

            // 禁忌过敏药品（既往史）
            drugRule.bill(bill.getAllergicmeds(), allergyMedicines, bill, billMsgList, NeoEnum.allergicmeds.getName());

            // 服用药品（主诉、现病史、既往史、结构化药品）
            drugRule.bill(bill.getOralmeds(), medicineAll, bill, billMsgList, NeoEnum.oralmeds.getName());

            // 禁忌人群
            groupRule.bill(wordCrfDTO, bill, billMsgList, NeoEnum.group.getName());

            // 禁用辅助检查描述
            commonRule.compareItemWithBill(bill.getPacsDesc(), pacsDescList, bill, billMsgList, NeoEnum.pacsDesc.getName());

            // 禁忌医疗器械及物品
            medEquRule.bill(wordCrfDTO, bill, billMsgList, NeoEnum.medEqu.getName());
        }

        // 24小时重复开单项
        commonRule.repeat24Bill(wordCrfDTO, billMsgList);

        // 互斥开单项
        commonRule.exclusionBill(billNeoMaxDTOList, wordCrfDTO, billMsgList);
        indicationDTO.setBillMsgList(billMsgList);
    }

}
