package com.diagbot.process;

import com.diagbot.dto.BillMsg;
import com.diagbot.dto.CriticalNeoDTO;
import com.diagbot.dto.IndicationDTO;
import com.diagbot.dto.WordCrfDTO;
import com.diagbot.enums.TypeEnum;
import com.diagbot.rule.LisRule;
import com.diagbot.rule.PacsRule;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.List;

/**
 * @Description: 危急值总入口
 * @author: zhoutg
 * @time: 2018/8/6 9:11
 */
@Component
public class CriticalProcess {

    @Autowired
    LisRule lisRule;
    @Autowired
    PacsRule pacsRule;

    /**
     * 处理业务
     *
     * @param criticalNeoDTOList
     * @param wordCrfDTO
     * @param res
     */
    public void process(List<CriticalNeoDTO> criticalNeoDTOList, WordCrfDTO wordCrfDTO, IndicationDTO res) {
        List<BillMsg> criticalList = res.getCriticalValList();
        for (CriticalNeoDTO critical : criticalNeoDTOList) {
            switch (TypeEnum.getEnum(critical.getType())) {
                case lis: // 化验
                    lisRule.critical(wordCrfDTO, critical, criticalList);
                    break;
                case pacs: // 辅检
                    pacsRule.critical(critical, criticalList);
                    break;
            }
        }
        res.setCriticalValList(criticalList);
    }
}
