package com.diagbot.process;

import com.diagbot.biz.push.entity.Lis;
import com.diagbot.dto.BillMsg;
import com.diagbot.dto.IndicationDTO;
import com.diagbot.dto.NodeNeoDTO;
import com.diagbot.dto.OtherTipNeoDTO;
import com.diagbot.dto.OtherTipPacsNeoDTO;
import com.diagbot.dto.OtherTipTransfusionNeoDTO;
import com.diagbot.dto.WordCrfDTO;
import com.diagbot.enums.NeoEnum;
import com.diagbot.enums.TypeEnum;
import com.diagbot.rule.CommonRule;
import com.diagbot.rule.GroupRule;
import com.diagbot.util.CoreUtil;
import com.diagbot.util.ListUtil;
import com.diagbot.util.MsgUtil;
import com.diagbot.util.StringUtil;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

/**
 * @Description: 其他提示总入口
 * @author: zhoutg
 * @time: 2018/8/6 9:11
 */
@Component
public class OtherTipProcess {

    @Autowired
    GroupRule groupRule;
    @Autowired
    CommonRule commonRule;

    /**
     * 处理业务——化验
     *
     * @param otherTipNeoDTOList
     * @param wordCrfDTO
     * @param res
     */
    public void processLis(List<OtherTipNeoDTO> otherTipNeoDTOList, WordCrfDTO wordCrfDTO, IndicationDTO res) {
        // 化验结果与诊断和开单药品进行匹配
        for (OtherTipNeoDTO otherTipNeoDTO : otherTipNeoDTOList) {
            List<NodeNeoDTO> factor = otherTipNeoDTO.getFactor();

            if (ListUtil.isNotEmpty(factor)) {
                for (NodeNeoDTO nodeNeoDTO : factor) {
                    Map map = new LinkedHashMap();
                    if (TypeEnum.disease.getName().equals(nodeNeoDTO.getTermtype())) { // 诊断
                        map = CoreUtil.compareNameWithNode(nodeNeoDTO, wordCrfDTO.getDiag());
                    } else if (TypeEnum.drug.getName().equals(nodeNeoDTO.getTermtype())) { // 药品
                        map = CoreUtil.compareNameWithNode(nodeNeoDTO, wordCrfDTO.getDrug());
                    } else if (NeoEnum.group.getName().equals(nodeNeoDTO.getTermtype())) { // "禁忌人群"
                        map = groupRule.gravidityRule(wordCrfDTO);
                    }
                    if (CoreUtil.getMapFlag(map) == true) {
                        String content = otherTipNeoDTO.getName() + otherTipNeoDTO.getDetailName();
                        if (StringUtil.isNotBlank(otherTipNeoDTO.getOtherValue())) {
                            content += otherTipNeoDTO.getOtherValue();
                        } else {
                            content += CoreUtil.subZeroAndDot(otherTipNeoDTO.getValue().toString());
                        }
                        BillMsg billMsg = MsgUtil.getCommonOtherTipLisMsg(content, (String)map.get("msg"), nodeNeoDTO.getTermtype());
                        res.getOtherList().add(billMsg);
                    }
                }
            }
        }
    }

    /**
     * 处理业务——辅检
     *
     * @param otherTipTransfusionNeoDTOList
     * @param res
     */
    public void processPacs(List<OtherTipPacsNeoDTO> otherTipTransfusionNeoDTOList, IndicationDTO res) {
        if (ListUtil.isNotEmpty(otherTipTransfusionNeoDTOList)) {
            // 去重
            Set<String> msgList = otherTipTransfusionNeoDTOList.stream().map(r -> r.getMsg()).collect(Collectors.toSet());
            for (String s : msgList) {
                BillMsg billMsg = MsgUtil.getCommonOtherTipPacsMsg(s);
                res.getOtherList().add(billMsg);
            }
        }
    }

    /**
     * 处理业务——输血
     *
     * @param otherTipTransfusionNeoDTOList
     * @param res
     */
    public void processTransfusion(List<OtherTipTransfusionNeoDTO> otherTipTransfusionNeoDTOList, IndicationDTO res) {
        for (OtherTipTransfusionNeoDTO bean : otherTipTransfusionNeoDTOList) {
            if (StringUtil.isBlank(bean.getName())) {
                continue;
            }
            StringBuffer sb = new StringBuffer();
            // 诊断
            if (ListUtil.isNotEmpty(bean.getDiag())) {
                sb.append("诊断");
                List<String> diag = bean.getDiag().stream().map(r -> r.getName()).collect(Collectors.toList());
                sb.append(StringUtils.join(diag, ","));
            }
            // 辅检结果
            if (ListUtil.isNotEmpty(bean.getPacsRes())) {
                List<String> pacsRes = bean.getPacsRes().stream().map(r -> r.getName()).collect(Collectors.toList());
                sb.append(StringUtils.join(pacsRes, ","));
            }
            // 化验
            if (ListUtil.isNotEmpty(bean.getLisList())) {
                for (Lis lis : bean.getLisList()) {
                    if (sb.toString().length() > 0) {
                        sb.append("，");
                    }
                    sb.append(lis.getName() + lis.getDetailName() + CoreUtil.getLisValue(lis));
                }
            }
            // 辅检名称
            if (ListUtil.isNotEmpty(bean.getPacs())) {
                if (sb.toString().length() > 0) {
                    sb.append("，");
                }
                sb.append("已开");
                List<String> pacs = bean.getPacs().stream().map(r -> r.getName()).collect(Collectors.toList());
                sb.append(StringUtils.join(pacs, ","));
            }
            // 手术
            if (ListUtil.isNotEmpty(bean.getOperation())) {
                List<String> operation = bean.getOperation().stream().map(r -> r.getName()).collect(Collectors.toList());
                if (sb.toString().length() > 0) {
                    sb.append("，");
                }
                sb.append("已开");
                sb.append(StringUtils.join(operation, ","));
            }
            BillMsg billMsg = MsgUtil.getCommonOtherTipTransfusionMsg(sb.toString(), bean.getName());
            res.getOtherList().add(billMsg);
        }
    }
}
