package com.diagbot.repository;

import com.diagbot.entity.node.Dept;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.data.neo4j.annotation.Query;
import org.springframework.data.neo4j.repository.Neo4jRepository;
import org.springframework.data.repository.query.Param;

import java.util.List;


public interface DeptRepository extends Neo4jRepository<Dept, Long> {

    List<Dept> findByNameContaining(String name);

    List<Dept> findByNameIs(String name);

    List<Dept> findByNameIn(List<String> names);

    Page<Dept> findByNameContainingIgnoreCaseOrPycodeContainingIgnoreCase(String name, String pycode, Pageable pageable);

    @Query("MATCH (n:`科室`) WHERE toLower(n.`name`) = toLower($name) \n" +
            "OR toLower(n.`拼音编码`) = toLower($pycode)  \n" +
            "RETURN n,[ (n)-[r]->(m) | [ r, m ] ], ID(n)\n" +
            "union\n" +
            "MATCH (n:`科室`) WHERE toLower(n.`name`) starts with toLower($name) \n" +
            "OR toLower(n.`拼音编码`) starts with toLower($pycode)  \n" +
            "WITH n SKIP 0 LIMIT $size\n" +
            "RETURN n,[ (n)-[r]->(m) | [ r, m ] ], ID(n)\n" +
            "union\n" +
            "MATCH (n:`科室`) WHERE toLower(n.`name`) CONTAINS toLower($name) \n" +
            "OR toLower(n.`拼音编码`) CONTAINS toLower($pycode)  \n" +
            "WITH n SKIP 0 LIMIT $size\n" +
            "RETURN n,[ (n)-[r]->(m) | [ r, m ] ], ID(n)\n")
    List<Dept> index(@Param("name") String name, @Param("pycode") String pycode, @Param("size") Integer size);


    @Query("MATCH (n:科室) return distinct(n.name) as name LIMIT $(size)")
    List<String> getDeptNames(@Param("size") Integer size);
}
