package com.diagbot.repository;

import com.diagbot.entity.node.PacsResult;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.data.neo4j.annotation.Query;
import org.springframework.data.neo4j.repository.Neo4jRepository;
import org.springframework.data.repository.query.Param;

import java.util.List;


public interface PacsResultRepository extends Neo4jRepository<PacsResult, Long> {

    List<PacsResult> findByNameIs(String name);

    List<PacsResult> findByNameContaining(String name);

    List<PacsResult> findByNameIn(List<String> names);
    @Query("match(f:辅助检查名称结果)-[r]->(d:医保疾病名称) where f.name in $pycode return d.name")
    List<String> getDisByPacsResults(@Param("pycode")List<String> names);

    Page<PacsResult> findByNameContainingIgnoreCaseOrPycodeContainingIgnoreCase(String name, String pycode, Pageable pageable);

    @Query("MATCH (n:`辅助检查名称`) WHERE n.`静态知识标识` = $isKL \n" +
            "AND (toLower(n.`name`) = toLower($name)  OR  toLower(n.`拼音编码`) = toLower($pycode))\n" +
            "RETURN n,[ (n)-[r]->(m) | [ r, m ] ], ID(n)\n" +
            "UNION\n" +
            "MATCH (n:`辅助检查名称`) WHERE n.`静态知识标识` = $isKL \n" +
            "AND (toLower(n.`name`) STARTS WITH toLower($name) OR toLower(n.`拼音编码`) STARTS WITH toLower($pycode))\n" +
            "WITH n SKIP 0 LIMIT $size\n" +
            "RETURN n,[ (n)-[r]->(m) | [ r, m ] ], ID(n)\n" +
            "UNION \n" +
            "MATCH (n:`辅助检查名称`) WHERE n.`静态知识标识` = $isKL  \n" +
            "AND (toLower(n.`name`) CONTAINS toLower($name) OR toLower(n.`拼音编码`) CONTAINS toLower($pycode))  \n" +
            "WITH n SKIP 0 LIMIT $size\n" +
            "RETURN n,[ (n)-[r]->(m) | [ r, m ] ], ID(n)\n")
    List<PacsResult> staticKnowledgeIndex(@Param("isKL") Integer isKL, @Param("name") String name, @Param("pycode") String pycode, @Param("size") Integer size);

    @Query("MATCH (n:`辅助检查名称`) WHERE toLower(n.`name`) = toLower($name) \n" +
            "OR toLower(n.`拼音编码`) = toLower($pycode)  \n" +
            "RETURN n,[ (n)-[r]->(m) | [ r, m ] ], ID(n)\n" +
            "union\n" +
            "MATCH (n:`辅助检查名称`) WHERE toLower(n.`name`) starts with toLower($name) \n" +
            "OR toLower(n.`拼音编码`) starts with toLower($pycode)  \n" +
            "WITH n SKIP 0 LIMIT $size\n" +
            "RETURN n,[ (n)-[r]->(m) | [ r, m ] ], ID(n)\n" +
            "union\n" +
            "MATCH (n:`辅助检查名称`) WHERE toLower(n.`name`) CONTAINS toLower($name) \n" +
            "OR toLower(n.`拼音编码`) CONTAINS toLower($pycode)  \n" +
            "WITH n SKIP 0 LIMIT $size\n" +
            "RETURN n,[ (n)-[r]->(m) | [ r, m ] ], ID(n)\n")
    List<PacsResult> index(@Param("name") String name, @Param("pycode") String pycode, @Param("size") Integer size);
}