package com.diagbot.repository;

import com.diagbot.entity.node.Symptom;
import org.springframework.data.neo4j.annotation.Query;
import org.springframework.data.neo4j.repository.Neo4jRepository;
import org.springframework.data.repository.query.Param;

import java.util.List;


public interface SymptomNameRepository extends Neo4jRepository<Symptom, Long> {

    List<Symptom> findByNameIs(String name);

    List<Symptom> findByNameIn(List<String> names);

    List<Symptom> findByNameContaining(String name);

    @Query("MATCH (n:`症状`) WHERE toLower(n.`name`) = toLower($name) \n" +
            "OR toLower(n.`拼音编码`) = toLower($pycode)  \n" +
            "RETURN n,[ (n)-[r]->(m) | [ r, m ] ], ID(n)\n" +
            "union\n" +
            "MATCH (n:`症状`) WHERE toLower(n.`name`) starts with toLower($name) \n" +
            "OR toLower(n.`拼音编码`) starts with toLower($pycode)  \n" +
            "WITH n SKIP 0 LIMIT $size\n" +
            "RETURN n,[ (n)-[r]->(m) | [ r, m ] ], ID(n)\n" +
            "union\n" +
            "MATCH (n:`症状`) WHERE toLower(n.`name`) CONTAINS toLower($name) \n" +
            "OR toLower(n.`拼音编码`) CONTAINS toLower($pycode)  \n" +
            "WITH n SKIP 0 LIMIT $size\n" +
            "RETURN n,[ (n)-[r]->(m) | [ r, m ] ], ID(n)\n")
    List<Symptom> index(@Param("name") String name, @Param("pycode") String pycode, @Param("size") Integer size);


    @Query("match(d:医保疾病名称)-[r{典型:1}]->(s) where any(label in labels(s) WHERE label in ['症状', '体征']) and s.name in {py} return d.name")
//    @Query("match(d:医保疾病名称)-[r]->(s) where any(label in labels(s) WHERE label in ['症状', '体征']) and s.name in {py} return d.name")
    List<String> getDisBySymptoms(@Param("py")List<String> symptoms);

    @Query("match(d:医保疾病名称)-[r{典型:1}]->(s:症状) where  s.name in {py} return d.name\n" +
            "union all\n" +
            "match(d:医保疾病名称)-[r{典型:1}]->(s:体征) where  s.name in {py} return d.name")
    List<String> getDisBySymptoms_1(@Param("py")List<String> symptoms);

    @Query("MATCH (n:症状) return distinct(n.name) as name LIMIT {size}")
    List<String> getSymptomNames(@Param("size") Integer size);
}

