package com.diagbot.repository;

import com.diagbot.entity.node.Symptom;
import com.diagbot.entity.node.Vital;
import org.springframework.data.neo4j.annotation.Query;
import org.springframework.data.neo4j.repository.Neo4jRepository;
import org.springframework.data.repository.query.Param;

import java.util.List;


public interface VitalRepository extends Neo4jRepository<Vital, Long> {

    List<Vital> findByNameIs(String name);

    List<Vital> findByNameIn(List<String> names);

    List<Vital> findByNameContaining(String name);

    @Query("MATCH (n:`体征`) WHERE toLower(n.`name`) = toLower($name) \n" +
            "OR toLower(n.`拼音编码`) = toLower($pycode)  \n" +
            "RETURN n,[ (n)-[r]->(m) | [ r, m ] ], ID(n)\n" +
            "union\n" +
            "MATCH (n:`体征`) WHERE toLower(n.`name`) starts with toLower($name) \n" +
            "OR toLower(n.`拼音编码`) starts with toLower($pycode)  \n" +
            "WITH n SKIP 0 LIMIT $size\n" +
            "RETURN n,[ (n)-[r]->(m) | [ r, m ] ], ID(n)\n" +
            "union\n" +
            "MATCH (n:`体征`) WHERE toLower(n.`name`) CONTAINS toLower($name) \n" +
            "OR toLower(n.`拼音编码`) CONTAINS toLower($pycode)  \n" +
            "WITH n SKIP 0 LIMIT $size\n" +
            "RETURN n,[ (n)-[r]->(m) | [ r, m ] ], ID(n)\n")
    List<Vital> index(@Param("name") String name, @Param("pycode") String pycode, @Param("size") Integer size);


    @Query("MATCH (n:体征) return distinct(n.name) as name LIMIT {size}")
    List<String> getVitalNames(@Param("size") Integer size);

}

