package com.diagbot.rule;

import com.diagbot.biz.push.entity.Item;
import com.diagbot.dto.BillMsg;
import com.diagbot.dto.BillNeoMaxDTO;
import com.diagbot.dto.NodeNeoDTO;
import com.diagbot.dto.WordCrfDTO;
import com.diagbot.util.CoreUtil;
import com.diagbot.util.MsgUtil;
import org.springframework.stereotype.Component;

import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * @description: 人群规则
 * @author: zhoutg
 * @time: 2020/8/3 14:47
 */
@Component
public class GroupRule {

    /**
     * 禁忌人群
     *
     * @param wordCrfDTO
     * @param billNeoMaxDTO
     * @return
     */
    public void bill(WordCrfDTO wordCrfDTO, BillNeoMaxDTO billNeoMaxDTO, List<BillMsg> billMsgList, String type) {
        List<NodeNeoDTO> group = billNeoMaxDTO.getGroup();
        Double age = wordCrfDTO.getAgeNum();
        for (NodeNeoDTO node : group) {
            // 妊娠目前从诊断里判断,包含“妊娠”
            String symptom = wordCrfDTO.getSymptom(); // 现病史内容
            String chief = wordCrfDTO.getChief(); // 主诉
            String menstrual = wordCrfDTO.getMenstrual(); // 月经史

            switch (node.getName()) {
                case "妊娠":
                    Map map = gravidityRule(wordCrfDTO);
                    if (CoreUtil.getMapFlag(map) == true) {
                        BillMsg commonBillMsg = MsgUtil.getCommonBillMsg(
                                billNeoMaxDTO.getOrderName(), billNeoMaxDTO.getOrderStandName(),
                                "妊娠", type, billNeoMaxDTO.getType());
                        billMsgList.add(commonBillMsg);
                    }
                    break;
                case "月经期":
                    String regex = "月经第(([零一二三四五六七八九十]{0,3})||([0-9]{0,2}))天";
                    Pattern pattern = Pattern.compile(regex);
                    Matcher matcher = pattern.matcher(symptom);
                    boolean rs = matcher.find();
                    Matcher matcher1 = pattern.matcher(menstrual);
                    boolean rs1 = matcher1.find();
                    if (rs || rs1 || symptom.contains("经期") || menstrual.contains("经期")) {
                        BillMsg commonBillMsg = MsgUtil.getCommonBillMsg(
                                billNeoMaxDTO.getOrderName(), billNeoMaxDTO.getOrderStandName(),
                                "月经期", type, billNeoMaxDTO.getType());
                        billMsgList.add(commonBillMsg);
                    }
                    break;
                case "幼儿":
                    if (age != null) {
                        if (age < 6.0D && 1.0D <= age) {
                            BillMsg commonBillMsg = MsgUtil.getCommonBillMsg(
                                    billNeoMaxDTO.getOrderName(), billNeoMaxDTO.getOrderStandName(),
                                    "幼儿", type, billNeoMaxDTO.getType());
                            billMsgList.add(commonBillMsg);
                        }
                    }
                    break;
                case "儿童":
                    if (age != null) {
                        if (age < 18.0D && age >= 6.0D) {
                            BillMsg commonBillMsg = MsgUtil.getCommonBillMsg(
                                    billNeoMaxDTO.getOrderName(), billNeoMaxDTO.getOrderStandName(),
                                    "儿童", type, billNeoMaxDTO.getType());
                            billMsgList.add(commonBillMsg);
                        }
                    }
                    break;
            }
        }
    }

    /**
     * 判断妊娠规则
     *
     * @param wordCrfDTO
     * @return
     */
    public Map gravidityRule(WordCrfDTO wordCrfDTO) {
        Map<String, Object> map = new LinkedHashMap<>();
        // 妊娠目前从诊断里判断,包含“妊娠”
        String symptom = wordCrfDTO.getSymptom(); // 现病史内容
        String chief = wordCrfDTO.getChief(); // 主诉
        // 主诉、现病史、诊断包含“孕妇”，“妊娠”
        boolean flag = false;
        if (symptom.contains("孕妇") || chief.contains("孕妇")
                || symptom.contains("妊娠") || chief.contains("妊娠")) {
            flag = true;
        } else {
            // 主诉、现病史出现孕几周
            String regex = ".*孕.{1,4}周.*"; // 孕xx周
            if (symptom.matches(regex) || chief.matches(regex)) {
                flag = true;
            } else {
                List<Item> diag = wordCrfDTO.getDiag();
                for (Item item : diag) {
                    // 诊断出现“孕几周”，孕字必须是第一个
                    String regex1 = "孕.{1,4}周.*"; // 孕xx周
                    if (item.getUniqueName().matches(regex1) || item.getUniqueName().contains("孕妇")
                            || item.getUniqueName().contains("妊娠")) {
                        flag = true;
                        break;
                    }
                }
            }
        }
        if (flag) {
            map.put("flag", flag);
            map.put("msg", "妊娠");
        }
        return map;
    }
}
