package com.diagbot.rule;

import com.diagbot.dto.BillMsg;
import com.diagbot.dto.BillNeoMaxDTO;
import com.diagbot.dto.CriticalNeoDTO;
import com.diagbot.dto.NodeNeoDTO;
import com.diagbot.enums.TypeEnum;
import com.diagbot.util.CoreUtil;
import com.diagbot.util.ListUtil;
import com.diagbot.util.MsgUtil;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Component;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @description: 疾病规则
 * @author: kwz
 * @time: 2020/8/3 14:47
 */
@Component
public class PacsRule {

    /**
     * 比较辅检——开单合理性
     *
     * @param nodeList
     * @param inputList
     * @return
     */
    public <T> void bill(List<NodeNeoDTO> nodeList, List<T> inputList, BillNeoMaxDTO billNeoMaxDTO, List<BillMsg> billMsgList, String type) {
        Map<String, String> old_stand = new HashMap<>();
        if (ListUtil.isNotEmpty(nodeList) && ListUtil.isNotEmpty(inputList)) {
            List<String> dl = nodeList.stream().map(x -> x.getName()).collect(Collectors.toList());
            for (T lis : inputList) {
                String val = (String) CoreUtil.getFieldValue(lis, "uniqueName");
                String val_name = (String) CoreUtil.getFieldValue(lis, "name");
                if (StringUtils.isNotBlank(val) && dl.contains(val)) {
                    old_stand.put(val_name, val);
                }
            }
        }
        if (old_stand.size() > 0) {
            for (Map.Entry<String, String> d : old_stand.entrySet()) {
                BillMsg commonBillMsg = MsgUtil.getCommonBillMsg(
                        billNeoMaxDTO.getOrderName(), billNeoMaxDTO.getOrderStandName(),
                        "已开" + d.getKey(), type, billNeoMaxDTO.getType());
                billMsgList.add(commonBillMsg);
            }
        }
    }

    /**
     * 比较辅检——危急值
     *
     * @param critical
     * @param criticalList
     */
    public void critical(CriticalNeoDTO critical, List<BillMsg> criticalList) {
        BillMsg commonBillMsg = MsgUtil.getCommonCriticalMsg(critical.getName(), TypeEnum.pacs.getName());
        criticalList.add(commonBillMsg);
    }
}
