package com.diagbot.util;
import com.diagbot.dto.BillMsg;
import com.diagbot.enums.NeoEnum;
import com.diagbot.enums.TypeEnum;

/**
 * @description: 提示语工具类
 * @author: zhoutg
 * @time: 2020/9/14 11:16
 */
public class MsgUtil {

    /**
     * 高危药品、手术通用提示信息
     *
     * @param name
     * @param highLevel
     * @param type
     * @return
     */
    public static BillMsg getCommonHighRiskMsg(String name, String highLevel, String type) {
        BillMsg billMsg = new BillMsg();
        if (TypeEnum.operation.getName().equals(type)) {
            billMsg.setMsg(name + "为高风险手术，请留意");
        } else if (TypeEnum.drug.getName().equals(type)) {
            billMsg.setMsg(name + "为" + highLevel + "药品，请留意");
        }
        billMsg.setType(type);
        return billMsg;
    }

    /**
     * 危急值通用提示信息
     *
     * @param content 匹配内容
     * @param type 类型
     * @return
     */
    public static BillMsg getCommonCriticalMsg(String content, String type) {
        BillMsg billMsg = new BillMsg();
        String msg = String.format("该患者%s，请留意", content);
        if (TypeEnum.pacs.getName().equals(type)) {
            msg = String.format("该患者检查结果%s，请留意", content);
        }
        billMsg.setMsg(msg);
        billMsg.setType(type);
        billMsg.setContent(content);
        return billMsg;
    }

    // /**
    //  * 开单合理性通用提示信息
    //  *
    //  * @param orderName 原开单项
    //  * @param orderStandName 标准开单项
    //  * @param content 匹配内容
    //  * @param type 类型
    //  * @return
    //  */
    // public static BillMsg getCommonBillMsg(String orderName, String orderStandName, String content, String type) {
    //     BillMsg billMsg = new BillMsg();
    //     String msg = String.format("该患者%s，不宜开%s", content, orderName);
    //     billMsg.setMsg(msg);
    //     billMsg.setOrderName(orderName);
    //     billMsg.setOrderStandName(orderStandName);
    //     billMsg.setContent(content);
    //     billMsg.setType(type);
    //     return billMsg;
    // }

    /**
     * 开单合理性通用提示信息
     *
     * @param orderName 原开单项
     * @param orderStandName 标准开单项
     * @param content 匹配内容
     * @param type 类型
     * @param type 开单项类型
     * @return
     */
    public static BillMsg getCommonBillMsg(String orderName, String orderStandName, String content, String type, String orderType) {
        BillMsg billMsg = new BillMsg();
        String msg = String.format("该患者%s，不宜开%s", content, orderName);
        // 输血提示语是另一种写法
        if (TypeEnum.transfusion.getName().equals(orderType)) {
            msg = String.format("该患者%s，谨慎输注%s", content, orderName);
        }
        billMsg.setMsg(msg);
        billMsg.setOrderName(orderName);
        billMsg.setOrderStandName(orderStandName);
        billMsg.setContent(content);
        billMsg.setType(type);
        return billMsg;
    }

    /**
     * 开单合理性——互斥提示信息
     *
     * @param complexOrderName 多个开单项
     * @param orderName 原开单项
     * @param content 匹配内容
     * @param type 类型
     * @return
     */
    public static BillMsg getBillExclusionMsg(String orderName, String content, String complexOrderName, String type) {
        BillMsg billMsg = new BillMsg();
        String msg = String.format("检查项目互斥：%s与%s不宜同时进行", orderName, content);
        billMsg.setMsg(msg);
        billMsg.setOrderName(complexOrderName);
        billMsg.setContent(content);
        billMsg.setType(type);
        return billMsg;
    }

    /**
     * 复杂情况高危手术提示
     *
     * @param content
     * @param name
     * @param type
     * @return
     */
    public static BillMsg getComplexOperationMsg(String name, String content, String type) {
        BillMsg billMsg = new BillMsg();
        String msg = String.format("该患者%s，%s为高风险手术，请留意", name, content);
        billMsg.setMsg(msg);
        billMsg.setContent(content);
        billMsg.setType(type);
        return billMsg;
    }

    /**
     * 其他提醒——化验
     *
     * @param content
     * @param name
     * @param type
     * @return
     */
    public static BillMsg getCommonOtherTipLisMsg(String content, String name, String type) {
        BillMsg billMsg = new BillMsg();
        String msg = "";
        if (TypeEnum.disease.getName().equals(type)) {
            msg = String.format("该患者%s，患有%s，请留意", content, name);
        } else if (TypeEnum.drug.getName().equals(type)) {
            msg = String.format("该患者%s，已开%s，请留意", content, name);
        } else if (NeoEnum.group.getName().equals(type)) {
            msg = String.format("该患者%s，%s，请留意", content, name);
        }
        billMsg.setMsg(msg);
        billMsg.setContent(content);
        billMsg.setType(type);
        return billMsg;
    }

    /**
     * 其他提醒——辅检
     *
     * @param content
     * @return
     */
    public static BillMsg getCommonOtherTipPacsMsg(String content) {
        BillMsg billMsg = new BillMsg();
        billMsg.setMsg(content);
        billMsg.setContent(content);
        return billMsg;
    }

    /**
     * 其他提醒——输血
     *
     * @param content
     * @param name
     * @return
     */
    public static BillMsg getCommonOtherTipTransfusionMsg(String content, String name) {
        BillMsg billMsg = new BillMsg();
        String msg = String.format("该患者%s，建议输注%s", content, name);
        billMsg.setMsg(msg);
        billMsg.setContent(content);
        return billMsg;
    }

    /**
     * 开单合理性24小时重复开通用提示信息
     *
     * @param orderName 原开单项
     * @param orderStandName 标准开单项
     * @param content 匹配内容
     * @param type 类型
     * @return
     */
    public static BillMsg getBillMsg24Repeat(String orderName, String orderStandName, String content, String type) {
        BillMsg billMsg = new BillMsg();
        String msg = String.format("%s重复开立", orderName);
        billMsg.setMsg(msg);
        billMsg.setOrderName(orderName);
        billMsg.setOrderStandName(orderStandName);
        billMsg.setContent(content);
        billMsg.setType(type);
        return billMsg;
    }

    /**
     * 开单合理性禁忌医疗器械及物品提示信息
     *
     * @param orderName 原开单项
     * @param orderStandName 标准开单项
     * @param content 匹配内容
     * @param type 类型
     * @return
     */
    public static BillMsg getBillMedEquMsg(String orderName, String orderStandName, String content, String type, String orderType) {
        BillMsg billMsg = new BillMsg();
        if (content.startsWith("具有")) {
            content = content.replaceFirst("具有", ""); // 将“具有”替换成空
        }
        String msg = String.format("该患者具有%s，不宜开%s", content, orderName);
        // 输血提示语是另一种写法
        if (TypeEnum.transfusion.getName().equals(orderType)) {
            msg = String.format("该患者具有%s，谨慎输注%s", content, orderName);
        }
        billMsg.setMsg(msg);
        billMsg.setOrderName(orderName);
        billMsg.setOrderStandName(orderStandName);
        billMsg.setContent(content);
        billMsg.setType(type);
        return billMsg;
    }

    /**
     * 开单合理性手术提示信息
     *
     * @param orderName 原开单项
     * @param orderStandName 标准开单项
     * @param content 匹配内容
     * @param type 类型
     * @return
     */
    public static BillMsg getCommonSurgeryMsg(String orderName, String orderStandName, String content, String type, String orderType) {
        BillMsg billMsg = new BillMsg();
        String msg = String.format("该患者有%s史，不宜开%s", content, orderName);
        // 输血提示语是另一种写法
        if (TypeEnum.transfusion.getName().equals(orderType)) {
            msg = String.format("该患者有%s史，谨慎输注%s", content, orderName);
        }
        billMsg.setMsg(msg);
        billMsg.setOrderName(orderName);
        billMsg.setOrderStandName(orderStandName);
        billMsg.setContent(content);
        billMsg.setType(type);
        return billMsg;
    }
}
