package com.diagbot.web;

import com.diagbot.dto.RespDTO;
import com.diagbot.dto.RetrievalDTO;
import com.diagbot.dto.StaticKnowledgeIndexDTO;
import com.diagbot.facade.ConceptFacade;
import com.diagbot.facade.RetrievalFacade;
import com.diagbot.vo.ConceptVO;
import com.diagbot.vo.RetrievalVO;
import com.diagbot.vo.StaticKnowledgeIndexVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.Valid;
import java.util.List;

/**
 * @Description:
 * @Author:zhaops
 * @time: 2020/8/12 16:45
 */
@RestController
@RequestMapping("/retrieval")
@Api(value = "术语检索相关API", tags = { "术语检索相关API" })
@SuppressWarnings("unchecked")
public class RetrievalController {
    @Autowired
    private RetrievalFacade retrievalFacade;
    @Autowired
    private ConceptFacade conceptFacade;

    @ApiOperation(value = "术语检索[zhaops]",
            notes = "type： 类型：1-化验大项、2-化验小项、3-辅检、4-诊断、5-药品、6-手术和操作、7-科室、8-输血、9-症状 <br>" +
                    "inputStr： 检索内容<br>" +
                    "sex： 性别：1-男、2-女、3-通用 <br>" +
                    "age： 年龄<br>")
    @PostMapping("/index")
    public RespDTO<RetrievalDTO> index(@Valid @RequestBody RetrievalVO retrievalVO) {
        RetrievalDTO data = retrievalFacade.index(retrievalVO);
        return RespDTO.onSuc(data);
    }

    @ApiOperation(value = "术语名称检索[Mark]",
            notes = "type： 类型：1-化验大项、2-化验小项、3-辅检、4-诊断、5-药品、6-手术和操作、7-科室、8-输血、9-症状、10-体征 <br>" +
                    "inputStr： 检索内容<br>" )
    @PostMapping("/namelist")
    public RespDTO<RetrievalDTO> namelist(@Valid @RequestBody RetrievalVO retrievalVO) {
        RetrievalDTO data = retrievalFacade.getname(retrievalVO);
        return RespDTO.onSuc(data);
    }

    @ApiOperation(value = "医学知识（静态知识）检索[zhaops]",
            notes = "types： 类型（多选）：0-全部、1-诊断、2-药品、3-检验、5-检查、6-手术和操作 <br>" +
                    "inputStr： 检索内容<br>")
    @PostMapping("/staticKnowledgeIndex")
    public RespDTO<List<StaticKnowledgeIndexDTO>> staticKnowledgeIndex(@Valid @RequestBody StaticKnowledgeIndexVO staticKnowledgeIndexVO) {
        List<StaticKnowledgeIndexDTO> data = retrievalFacade.staticKnowledgeIndex(staticKnowledgeIndexVO);
        return RespDTO.onSuc(data);
    }

    @ApiOperation(value = "术语批量查询[zhaops]",
            notes = "type： 类型：1-化验大项、2-化验小项、3-辅检、4-诊断、5-药品、6-手术和操作、7-科室、8-输血 <br>" +
                    "names： 术语列表<br>")
    @PostMapping("/getConceptNames")
    public RespDTO<List<String>> getConceptNames(@Valid @RequestBody ConceptVO conceptVO) {
        List<String> data = conceptFacade.getConceptNames(conceptVO);
        return RespDTO.onSuc(data);
    }
}