package com.diagbot.web;

import com.diagbot.dto.RespDTO;
import com.diagbot.dto.StandConvertCrfBatchDTO;
import com.diagbot.dto.StandConvertCrfDTO;
import com.diagbot.facade.TestFacade;
import com.diagbot.vo.StandConvertCrfVO;
import com.diagbot.vo.TestAllVO;
import com.diagbot.vo.TestLineVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletResponse;
import java.util.List;
import java.util.Map;

/**
 * @Description: CDSS核心测试API控制层
 * @author: zhoutg
 * @time: 2018/8/30 10:12
 */
@RestController
@RequestMapping("/test")
@Api(value = "测试API", tags = { "测试API" })
@SuppressWarnings("unchecked")
public class TestController {

    @Autowired
    private TestFacade testFacade;

    @ApiOperation(value = "开单合理性测试API[zhoutg]",
            notes = "idNum：指定行测试<br>" +
                    "sheetIndex：sheet的下标，默认为0，表示第一个sheet")
    @PostMapping("/testIndication")
    public RespDTO<Map<String, Object>> testIndication(@RequestParam("file") MultipartFile file, TestLineVO testLineVO) {
        return RespDTO.onSuc(testFacade.importExcel(file, testLineVO));
    }

    @ApiOperation(value = "导出[zhoutg]",
            notes = "导出")
    @PostMapping("/testExport")
    public void testExport(HttpServletResponse response) {
        testFacade.exportExcel(response);
    }

    @ApiOperation(value = "开单合理性【输血】测试API[zhoutg]",
            notes = "idNum：指定行测试<br>" +
                    "sheetIndex：sheet的下标，默认为0，表示第一个sheet")
    @PostMapping("/testIndicationTransfusion")
    public RespDTO<Map<String, Object>> testIndicationTransfusion(@RequestParam("file") MultipartFile file, TestLineVO testLineVO) {
        return RespDTO.onSuc(testFacade.testIndicationTransfusion(file, testLineVO));
    }

    @ApiOperation(value = "危急值测试API——化验[zhoutg]",
            notes = "idNum：指定行测试<br>" +
                    "sheetIndex：sheet的下标，默认为0，表示第一个sheet")
    @PostMapping("/testCriticalLis")
    public RespDTO<Map<String, Object>> testCriticalLis(@RequestParam("file") MultipartFile file, TestLineVO testLineVO) {
        return RespDTO.onSuc(testFacade.importCriticalLisExcel(file, testLineVO));
    }

    @ApiOperation(value = "危急值测试API——辅检[zhoutg]",
            notes = "idNum：指定行测试<br>" +
                    "sheetIndex：sheet的下标，默认为0，表示第一个sheet")
    @PostMapping("/testCriticalPacs")
    public RespDTO<Map<String, Object>> testCriticalPacs(@RequestParam("file") MultipartFile file, TestLineVO testLineVO) {
        return RespDTO.onSuc(testFacade.importCriticalPacsExcel(file, testLineVO));
    }

    @ApiOperation(value = "高危手术测试API[zhoutg]",
            notes = "idNum：指定行测试<br>" +
                    "sheetIndex：sheet的下标，默认为0，表示第一个sheet")
    @PostMapping("/testHighRiskOperation")
    public RespDTO<Map<String, Object>> testHighRiskOperation(@RequestParam("file") MultipartFile file, TestLineVO testLineVO) {
        return RespDTO.onSuc(testFacade.testHighRiskOperationExcel(file, testLineVO));
    }

    @ApiOperation(value = "高危药品测试API[zhoutg]",
            notes = "idNum：指定行测试<br>" +
                    "sheetIndex：sheet的下标，默认为0，表示第一个sheet")
    @PostMapping("/testHighRiskDrug")
    public RespDTO<Map<String, Object>> testHighRiskDrug(@RequestParam("file") MultipartFile file, TestLineVO testLineVO) {
        return RespDTO.onSuc(testFacade.testHighRiskDrugExcel(file, testLineVO));
    }

    @ApiOperation(value = "其他值提醒测试——化验[zhoutg]",
            notes = "idNum：指定行测试<br>" +
                    "sheetIndex：sheet的下标，默认为0，表示第一个sheet")
    @PostMapping("/testOtherTipLis")
    public RespDTO<Map<String, Object>> testOtherTipLis(@RequestParam("file") MultipartFile file, TestLineVO testLineVO) {
        return RespDTO.onSuc(testFacade.testOtherTipLis(file, testLineVO));
    }

    @ApiOperation(value = "其他值提醒测试——辅检[zhoutg]",
            notes = "idNum：指定行测试<br>" +
                    "sheetIndex：sheet的下标，默认为0，表示第一个sheet")
    @PostMapping("/testOtherTipPacs")
    public RespDTO<Map<String, Object>> testOtherTipPacs(@RequestParam("file") MultipartFile file, TestLineVO testLineVO) {
        return RespDTO.onSuc(testFacade.testOtherTipPacs(file, testLineVO));
    }

    @ApiOperation(value = "其他值提醒测试——输血API[zhoutg]",
            notes = "idNum：指定行测试<br>" +
                    "sheetIndex：sheet的下标，默认为0，表示第一个sheet")
    @PostMapping("/testOtherTipTransfusion")
    public RespDTO<Map<String, Object>> testOtherTipTransfusion(@RequestParam("file") MultipartFile file, TestLineVO testLineVO) {
        return RespDTO.onSuc(testFacade.testOtherTipTransfusion(file, testLineVO));
    }

    @ApiOperation(value = "标准词转换API[zhoutg]", notes = "类型，疾病: disease，症状: symptom，手术和操作:operation，药品: drug，实验室检查：lis，辅助检查：pacs， 辅助检查：vital")
    @PostMapping("/testStandConvert")
    public RespDTO<StandConvertCrfDTO> testStandConvert(@RequestBody StandConvertCrfVO standConvertCrfVO) {
        return RespDTO.onSuc(testFacade.testStandConvert(standConvertCrfVO));
    }

    @ApiOperation(value = "标准词批量转换API[zhoutg]", notes = "类型，疾病: disease，症状: symptom，手术和操作:operation，药品: drug，实验室检查：lis，辅助检查：pacs， 查体：vital")
    @PostMapping("/testStandConvertBatch")
    public RespDTO<StandConvertCrfBatchDTO> testStandConvertBatch(@RequestBody List<StandConvertCrfVO> standConvertCrfVOList) {
        return RespDTO.onSuc(testFacade.testStandConvertBatch(standConvertCrfVOList));
    }

    @ApiOperation(value = "复杂高危手术测试API[zhoutg]",
            notes = "idNum：指定行测试<br>" +
                    "sheetIndex：sheet的下标，默认为0，表示第一个sheet")
    @PostMapping("/testHighRiskProcess")
    public RespDTO<Map<String, Object>> testHighRiskProcess(@RequestParam("file") MultipartFile file, TestLineVO testLineVO) {
        return RespDTO.onSuc(testFacade.testHighRiskProcess(file, testLineVO));
    }

    @ApiOperation(value = "批量测试[zhoutg]", notes = "参数：目录所在路径，示例：D:\\newSVN\\2020新版CDSS\\05.其他资料\\数据")
    @PostMapping("/testAll")
    public RespDTO<Map<String, Object>> testAll(TestAllVO testAllVO) {
        return RespDTO.onSuc(testFacade.testAll(testAllVO));
    }

    @ApiOperation(value = "导入开单合理性文档[Mark]", notes = "")
    @PostMapping("/importBill")
    public RespDTO<List<Map<String, Object>>> importBill(@RequestParam("file") MultipartFile file, TestLineVO testLineVO) {
        return RespDTO.onSuc(testFacade.importBill(file, testLineVO));
    }
}

